// VivaGraph layout variable.
Graph.prototype.layout = undefined;

// Called when node position change.
// @param [node] VivaGraphsJS Node element.
// @param [silent] If true do not send event in C# side.
Graph.prototype.onNodeChanged = function(node, silent) {
	var pos = this.layout.getNodePosition(node.id);
	var isSilent = (silent == true) ? true : false;

	if(window.external == 'Dataedo.Tools.ERD.ERDEditor') {
		window.external.NotifyNodePosition(node.id, pos.x, pos.y, isSilent);
	}
}

// Find (and set) best position for unpinned nodes.
Graph.prototype.arrangeNodes = function() {
	for (var i = 0; i < 100; ++i) {
  		this.layout.step();
    }

    var that = this;
    this.graph.forEachNode(function(node) {
    	that.onNodeChanged(node, true);
    });
}

// Add node.
// @param [id] Node id.
// @param [userData.label] <string> Text to show on entity.
//        [userData.deleted] <bool> Specifies whether the node is removed or not.
// 		                            If true, the node style will change.
//        [userData.type] <string> Type of entity (table, view).
Graph.prototype.addNode = function(id, options) {
	// Parse label
	if(typeof options.label !== 'string') {
		options.label = "";
	}

	// Parse type
	var allowedTypes = ['table', 'view'];
	if(typeof options.type !== 'string'
	|| !$.inArray(options.type, allowedTypes)) {
		options.type = 'table';
	}

	// Create node
	var node = this.graph.addNode(id, options);

	// Set default position
	if($.isArray(options.pos)
	&& typeof options.pos[0] === 'number'
	&& typeof options.pos[1] === 'number') {
		this.setNodePos(node.id, options.pos[0], options.pos[1]);
	}
}

// Change node position.
// @param [nodeId] Node id to be changed.
// @param [x] New x position of node, or null to leave unchanged.
// @param [y] New y position of node, or null to leave unchanged.
Graph.prototype.setNodePos = function(nodeId, x, y) {
	var pos = this.layout.getNodePosition(nodeId);
	var node = this.graph.getNode(nodeId);

	var newX = x == null ? pos.x : x;
	var newY = y == null ? pos.y : y;

	this.layout.setNodePosition(nodeId, newX, newY);
	this.layout.pinNode(node, true);
}

// Lock node position.
// @param [nodeId] Node id.
Graph.prototype.pinNode = function(nodeId) {
	var node = this.graph.getNode(nodeId);
	this.layout.pinNode(node, true);
}

// Lock position for specified nodes.
// @param nodesId Array of nodes id, or null to pin all.
Graph.prototype.pinNodes = function(nodesId) {
	var self = this;
	
	// Pin specified nodes:
	if($.isArray(nodesId) && nodesId.length > 0) {
		return nodesId.forEach(this.pinNode);
	}

	// Not set - pin all:
	this.graph.forEachNode(function(node) {
    	self.layout.pinNode(node, true);
    });
}