var path = {
    rect: function (x, y, width, height) {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
    },

    point: function (x, y) {
        this.x = x;
        this.y = y;
    },

    zoneSafeDistance: 5,

    pointZoneX: function(point, blocks) {
        var zone = {
            min: -Infinity,
            max: Infinity
        };

        blocks.forEach(function (block) {
            if (block.y > point.y
            && block.y + block.heigt < point.y) {
                return;
            }

            // On the left side
            if (block.x + block.width < point.x) {
                var barrier = block.x + block.width + path.zoneSafeDistance;
                Math.max(barrier, zone.min);
                return;
            }

            // On the right side
            if (block.x > point.x) {
                var barrier = block.x - path.zoneSafeDistance;
                Math.min(barrier, zone.max);
                return;
            }
        });

        return zone;
    },

    pointZoneY: function(point, blocks) {
        var zone = {
            min: -Infinity,
            max: Infinity
        };

        blocks.forEach(function (block) {
            if (block.x > point.x
            && block.x + block.width < point.x) {
                return;
            }

            // Above
            if (block.y + block.height < point.y) {
                var barrier = block.y + block.height + zoneSafeDistance;
                Math.max(barrier, zone.min);
                return;
            }

            // Belowe
            if (block.y > point.y) {
                var barrier = block.y - zoneSafeDistance;
                Math.min(barrier, zone.max);
                return;
            }
        });

        return zone;
    },

    findPath: function (pointA, pointB, blocks) {
        var depthLimit = 3;
        return path.depthSearchPath(pointA, pointB, blocks, depthLimit);
    },

    /*getAvailablePoints: function(pointA, pointB, zone, blocks) {

    },*/

    depthSearchPath: function(pointA, pointB, blocks, depth) {
        // Too long path
        if(--depth < 0) {
            return [];
        }

        // Path match
        if(pointA.x == pointB.x
        && pointA.y == pointB.y) {
            //console.log('win');
            //console.log('('+ pointA.x +', '+ pointA.y +')');
            return [pointA];
        }

        // Iterate possibilities moves x
        var zone = path.pointZoneX(pointA, blocks);
        var paths = [];

        var p = new path.point((pointB.x + pointA.x)/2, pointA.y);
        if(p.x >= zone.min
        && p.x <= zone.max) {
            var r = path.depthSearchPath(p, pointB, blocks, depth);
            if(r.length > 0) {
                paths.push(r.concat(pointA));
                //console.log('('+ pointA.x +', '+ pointA.y +')');
            }
        }

        var p = new path.point(pointB.x, pointA.y);
        if(p.x >= zone.min
        && p.x <= zone.max) {
            var r = path.depthSearchPath(p, pointB, blocks, depth);
            if(r.length > 0) {
                paths.push(r.concat(pointA));
                //console.log('('+ pointA.x +', '+ pointA.y +')');
            }
        }

        blocks.forEach(function(block) {
            var p = new path.point(block.x - path.zoneSafeDistance, pointA.y);
            if(p.x >= zone.min
            && p.x <= zone.max) {
                var r = path.depthSearchPath(p, pointB, blocks, depth);
                if(r.length > 0) {
                    paths.push(r.concat(pointA));
                //console.log('('+ pointA.x +', '+ pointA.y +')');
                }
            }

            var p = new path.point(block.x + block.width + path.zoneSafeDistance, pointA.y);
            if(p.x >= zone.min
            && p.x <= zone.max) {
                var r = path.depthSearchPath(p, pointB, blocks, depth);
                if(r.length > 0) {
                    paths.push(r.concat(pointA));
                //console.log('('+ pointA.x +', '+ pointA.y +')');
                }
            }
        });

        // Iterate possibilities moves y
        var p = new path.point(pointA.x, (pointB.y + pointA.y)/2);
        if(p.x >= zone.min
        && p.x <= zone.max) {
            var r = path.depthSearchPath(p, pointB, blocks, depth);
            if(r.length > 0) {
                paths.push(r.concat(pointA));
                //console.log('('+ pointA.x +', '+ pointA.y +')');
            }
        }

        var p = new path.point(pointA.x, pointB.y);
        if(p.y >= zone.min
        && p.y <= zone.max) {
            var r = path.depthSearchPath(p, pointB, blocks, depth);
            if(r.length > 0) {
                paths.push(r.concat(pointA));
                //console.log('('+ pointA.x +', '+ pointA.y +')');
            }
        }

        blocks.forEach(function(block) {
            var p = new path.point(pointA.x, block.y - path.zoneSafeDistance);
            if(p.y >= zone.min
            && p.y <= zone.max) {
                var r = path.depthSearchPath(p, pointB, blocks, depth);
                if(r.length > 0) {
                    paths.push(r.concat(pointA));
                //console.log('('+ pointA.x +', '+ pointA.y +')');
                }
            }

            var p = new path.point(pointA.x, block.y + block.height + path.zoneSafeDistance);
            if(p.y >= zone.min
            && p.y <= zone.max) {
                var r = path.depthSearchPath(p, pointB, blocks, depth);
                if(r.length > 0) {
                    paths.push(r.concat(pointA));
                    //console.log('('+ pointA.x +', '+ pointA.y +')');
                }
            }
        });

        // Return best path
        paths.sort(function(a, b) {
            return a.length - b.length;
        });

        return paths[0] != undefined ? paths[0] : [];
    },
}